#! /usr/bin/Rscript
###
### Simulation showing how number of seeds and correlation affect 
### sgm performance in terms of match ratio.
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(pryr)
require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(VN)

run <- FALSE

if(run){
    ## If running on laptop: 
    #require(doMC); nCores <- 4
    ## If running on cluster computer (MARCC)
    require(doMPI)
    source("../fig03/randseedselect.r")
    source("sgm.ordered.whungct.R")
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    B = matrix(c(0.7,0.3,0.4,0.3,0.7,0.3,0.4,0.3,0.7),3,byrow=TRUE)
    #B1 <- matrix(c(0.7,0.1,0.2,0.1,0.7,0.1,0.2,0.1,0.7),3,byrow=TRUE)
    n = rep(100,3) #c(75,75,75)
    
    np<-c(0,n)
    P <- matrix(0,sum(n),sum(n))
    for(ip in 1:length(n)){
        for(j in 1:length(n)){
            P[(sum(np[1:ip])+1):sum(np[1:ip+1]),
              (sum(np[1:j])+1):sum(np[1:j+1])] <-
                B[ip,j]*matrix(1,np[ip+1],np[j+1])
        }
    }
    rho <- seq(0,1,by=.1)
    mvary <- seq(0,20,by=1)
    paird <- expand.grid(m=mvary,rho=rho)
    lenprd <- nrow(paird)
    
    iter <- 20
    len2 <- 150 #500 #docs ending in 500 indicate this - otherwise, used len2=300
    #K <- 50
    
    sumn <- sum(n)
    set.seed(3067)
    SS <- matrix(sample(1e6,len2*lenprd,replace=FALSE),nrow=lenprd,ncol=len2)
    
    
    MK <- foreach(i=1:lenprd) %:% 
          foreach(le=1:len2, .combine='rbind') %dopar% {
            s <- paird[i,1]
            seeds <- cbind(1:s,1:s)
            corr <- paird[i,2]
            J <- matrix(1,(sumn-s),(sumn-s)); # J
            bc <- (1/(sumn-s))*J #this is the barycenter
    
            setseed <- 12345*le
            set.seed(SS[i,le])
    
            ### generate random correlated matrices 
    	### -- note: Bcorr corresponds to first (nrow(B)) vertices of A
            RM <- randmatcorr(B,n,n,P,corr,setseed)
            RMs <- randseedselect(RM$A,RM$Bcorr,s)
            A <- RMs$A
            Bcorr <- RMs$Bc
    
            ### create names for matrices in form gi_corr_s.txt -- 
            ### corr*10 is used for ease of not having multiple decimals in
            ### file name
            fa <- sprintf("AdjMats/A%03g_%02g_%02g.txt",le,corr*10,s)
            fb <- sprintf("AdjMats/B%03g_%02g_%02g.txt",le,corr*10,s)
            write.table(A, file=fa, row.names=FALSE, col.names=FALSE)
            write.table(Bcorr, file=fb, row.names=FALSE, col.names=FALSE)
    	
    	### Use sgm to match graphs
            stime <- system.time(
                memdiff <- mem_change(
    		# If don't care about hung iteration count use sgm fcn
    		# doesn't need bc, and allows for seeds not to be first m vertices
                    #Psall <- sgm(A,Bcorr,seeds,start="barycenter",iteration=iter)
    
                    #if calculating bc and have first s vertices as seeds
                    #Psall <- sgm.ordered(A,Bcorr,s,bc,pad=0,iteration=iter)
    
    		Psall <- sgm.ordered.whungct(A,Bcorr,s,bc,pad=0,iteration=iter)
                )
            )
            memused <- mem_used()
            Ps <- Psall$P        
            hungit <- Psall$hungit
    
            frob <- norm(A%*%Ps - Ps%*%Bcorr,type="F")
    
    	### Determine prop of correctly matched nodes (not including seeds)	
    	match <- (sum(diag(Ps))-s)/(nrow(Ps)-s)
            #c(voiLocK, maxvoiLoc, sizenbdA, sizenbdBcorr,length(propShared))
    	#match
    
            ## performance, runtime (elapsed), memory, FrobNorm, HungIters
    	cbind(match,stime[3],memused,memdiff,frob,hungit)
    }
    
    #save.image(file="sgm12noplot_mrhovary_n225.RData")

    MKbar <- as.numeric(lapply(MK,function(x){mean(x[,1])}))
    MKsd <- as.numeric(lapply(MK,function(x){sd(x[,1])/sqrt(len2)}))
    MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2
    df1 <- data.frame(paird,MKbar, MKci)
}else{
    load(file="sgm12withplot_mrhovary_n300.RData")
}



p04 <- 
        ggplot(data=df1, aes(x=m, y = MKbar,colour=as.factor(rho))) + 
        geom_line(aes(colour=as.factor(rho)),size=1)  + 
        geom_point() + 
        theme_classic() + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.7)) + 
        theme(text=element_text(size=25)) + 
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab(expression(paste("number of seeds ",m))) +
        guides(color=guide_legend(title=expression(rho))) #+
        #labs(title= expression(paste("SGM: Vary m and ",rho)))

#pdf("sgm12_mrhovary_n300.pdf",height=6,width=8)
p04
#dev.off()

if(run){
    #save.image(file="sgm12withplot_mrhovary_n300.RData")
    
    
    if(exists("cl")){
            closeCluster(cl)
    	mpi.quit()
    	}
}
    
#   Time:
##  Working status:
### Comments:
####Soli Deo Gloria
