#! /usr/bin/Rscript
###
### Illustration of how the padding affects the matching in SGM.
###
### Heather Gaddy Patsolic 
### 2016 <hgaddy1@jhu.edu>
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(VN)

source("./randseedselect.r")
source("./naivesgm.r")


B = matrix(c(0.9,0.4,0.3,0.2,0.4,0.9,0.4,0.3,0.3,0.4,0.9,0.4,0.2,0.3,0.4,0.7),4,byrow=TRUE)
n = rep(100,4) 
m = rep(75,4)

np<-c(0,n)
P <- matrix(0,sum(n),sum(n))
for(ip in 1:length(n)){
    for(j in 1:length(n)){
        P[(sum(np[1:ip])+1):sum(np[1:ip+1]),
          (sum(np[1:j])+1):sum(np[1:j+1])] <-
            B[ip,j]*matrix(1,np[ip+1],np[j+1])
    }
}
rho <- 0.5
mvary <- 10 
paird <- expand.grid(m=mvary,rho=rho)
lenprd <- nrow(paird)

iter <- 20
len2 <- 1

sumn <- sum(n)
set.seed(3067)
SS <- matrix(sample(1e6,len2*lenprd,replace=FALSE),nrow=lenprd,ncol=len2)

i<-1; le <- 1
        s <- paird[i,1]
        corr <- paird[i,2]
        J <- matrix(1,(sumn-s),(sumn-s)); # J
        bc <- (1/(sumn-s))*J #this is the barycenter

        setseed <- 12345*le
        set.seed(SS[i,le])

        ### generate random correlated matrices 
	### -- note: Bcorr corresponds to first (nrow(B)) vertices of A
        RM <- randmatcorr(B,n,m,P,corr,setseed)
        RMs <- randseedselect(RM$A,RM$Bcorr,s)
        A <- RMs$A
        seeds <- RMs$seeds

        nseeds <- setdiff(1:400,seeds)
        nbc <- setdiff(nseeds,1:300)
        si <- sort(c(seeds,nseeds),index.return=TRUE)
        Aunperm1 <- A[si$ix,si$ix]
        indx <- c(1:75,301:325,76:150,326:350,151:225,351:375,226:300,376:400)
        Aunperm <- Aunperm1[indx,indx]

        Bcorr <- RMs$Bc
        z12 <- matrix(0,nrow=300,ncol=100)
        z22 <- matrix(0,nrow=100,ncol=400)
        Bcnaive <- cbind(Bcorr,z12);
        Bcnaive <- rbind(Bcnaive,z22)

        restid <- c(76:100,176:200,276:300,376:400)
        Bunperm1 <- Bcnaive[si$ix,si$ix]
        Bunperm2 <- Bunperm1[indx,indx]
        Bunperm2[restid,] <- 1.5
        Bunperm2[,restid] <- 1.5
        Bunperm1[301:400,] <- 1.5
        Bunperm1[,301:400] <- 1.5
	
        plot(raster(Aunperm))
        plot(raster(Bunperm1))
        plot(raster(Bunperm2))
        

        Bextra <- adjcorrH(Aunperm,P,corr)
        Bextra1 <- Bunperm2
        Bextra1[restid,] <- Bextra[restid,]
        Bextra1[,restid] <- Bextra[,restid]
        
        plot(raster(Bextra1))
        
        


        #####################

	### Use sgm to match graphs
        seeds <- cbind(1:s,1:s)
        Psadopted <- sgm(A,Bcorr,seeds)
        Psa <- Psadopted$P        

        Psnaive <- naivesgm(A,Bcorr,s,start=bc,iteration=iter)
        Psn <- Psnaive$P

        PaP <- Psa%*%Bcnaive%*%t(Psa)
        PaP <- PaP[si$ix,si$ix]
        PaP <- PaP[indx,indx]
        pada <- which(apply(PaP,1,sum)==0)
        PaP[pada,] <- 1.5
        PaP[,pada] <- 1.5
        plot(raster(PaP))

        PnP <- Psn%*%Bcnaive%*%t(Psn)
        PnP <- PnP[si$ix,si$ix]
        PnP <- PnP[indx,indx]
        padn <- which(apply(PnP,1,sum)==0)
        PnP[padn,] <- 1.5
        PnP[,padn] <- 1.5
        plot(raster(PnP))

        rasterAfull <- raster(Bextra1)
        rasterB <- raster(Aunperm)
        rasterAdeletes <- raster(Bunperm2)
        rasterA <- raster(Bunperm1)
        rasteradopt <- raster(PaP)
        rasternaive <- raster(PnP)



pdf("Afull.pdf",height=7.9,width=8)
raster::plot(rasterAfull ,col=c("white","black"),axes=F,legend=F)
dev.off()

pdf("Bfull.pdf",height=7.9,width=8)
raster::plot(rasterB ,col=c("white","black"),axes=F,legend=F)
dev.off()

pdf("Adeletes.pdf",height=7.9,width=8)
raster::plot(rasterAdeletes ,col=c("white","black","purple"),axes=F,legend=F)
dev.off()

pdf("Asub.pdf",height=7.9,width=8)
raster::plot(rasterA ,col=c("white","black","lightgreen"),axes=F,legend=F)
dev.off()

pdf("adopted.pdf",height=7.9,width=8)
raster::plot(rasteradopt ,col=c("white","black","lightgreen"),axes=F,legend=F)
dev.off()

pdf("naive.pdf",height=7.9,width=8)
raster::plot(rasternaive ,col=c("white","black","lightgreen"),axes=F,legend=F)
dev.off()


##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
