#' Generates a pair of adjacency matrices for 
#'          graphs from a \eqn{rho}-correlated
#'          SBM with s seeds randomly selected 
#'          and moved to first s row/col's
#'          if graphs differ in size, assumes first vertices correspond
#'
#' @param  B block model matrix of probabilities
#' @param  n vector of sizes of blocks in main matrix A
#' @param  m vector of sizes of corresponding matrix created from A
#' @param  s number of seeds
#' @param  P matrix of probabilities size of sum(n)
#' @param  corr correlation of edge in B v. A (should be positive)
#' @param  setseed True, then set seed. False, then doesn't
#'
#' @return  A  the primary adjacency matrix A with first s vertices
#'          seeds
#' @return  Bcor the corresponding matrix from A with first s vertices
#'          seeds
#'
#' @author Heather Gaddy Patsolic <hgaddy1@jhu.edu>
#' @export

randseedselect <- function(A,B,s){
    n <- nrow(A)
    m <- nrow(B)
    seeds <- sample(1:min(n,m),s,replace=FALSE)
    nsA <- setdiff(1:n,seeds)
    nsB <- setdiff(1:m,seeds)
    A <- A[c(seeds,nsA),c(seeds,nsA)]
    Bc <- B[c(seeds,nsB),c(seeds,nsB)]
    L <- list(A=A,Bc=Bc,seeds=seeds)
    return(L)
} # END FUNCTION


#   Time:
##  Working status:
### Comments:
####Soli Deo Gloria
