#! /usr/bin/Rscript
###
### Exploring whether or not the restarts are beneficial.
### In particular, for 50 pairs of 0.5-correlated graphs we run 
### softSGM with R=100, and report the average number of correctly 
### matched vertices at top K for K = 1:50.
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(VN)
source("randseedselect.r")

require(doMC); nCores <- 4
#require(doMPI)

## Register Cores for parallelism 
if("package:doMPI" %in% search()){
	cl <- startMPIcluster()
	registerDoMPI(cl)} else {
	registerDoMC(nCores)
	}



B = matrix(c(0.7,0.3,0.4,0.3,0.7,0.3,0.4,0.3,0.7),3,byrow=TRUE)
n = c(100,100,100)
m = c(100,100,100)

np<-c(0,n)
P <- matrix(0,sum(n),sum(n))
for(i in 1:length(n)){
    for(j in 1:length(n)){
        P[(sum(np[1:i])+1):sum(np[1:i+1]),
          (sum(np[1:j])+1):sum(np[1:j+1])] <-
            B[i,j]*matrix(1,np[i+1],np[j+1])
    }
}
corr <- 0.5

s <- 1 # number of seeds for seeded graph matching
g <- 0.1 # gamma <- max tolerance for distance from barycenter
R <- 100
iter <- 20
len2 <- 50
K <- 50
set.seed(306)
SS <- sample(1e6,len2,replace=FALSE)

sumn <- sum(n)

MK <- foreach(le=1:len2, .combine='rbind') %dopar% {
    setseed <- 12345*le
    set.seed(SS[le])
    RM <- randmatcorr(B,n,m,P,corr,setseed)
    RMs <- randseedselect(RM$A,RM$Bcorr,s)
    A <- RMs$A
    Bcorr <- RMs$Bc

    Ps <- multiStart(A,Bcorr,R,s,g)

    ### Determine prop of correctly matched nodes (not including seeds)	
    rankd <- pass2ranksuplus(Ps)
    matchtopk <- rep(NA,K)
    for(i in 1:K){
        matchtopk[i] <- sum(diag(rankd) <= i)
    }
    match <- (matchtopk-s)/(nrow(Ps)-s)
#    MK[le,] <- match
    match
}

# MK displays how many correctly matched looking at top k for k in 1:K
MKbar <- apply(MK,2,mean)
MKse <- apply(MK,2,sd)/sqrt(len2)
MKci <- cbind(c(MKbar-2*MKse),c(MKbar+2*MKse)) # for 95% CI, use 1.96 instead of 2

dfr5 <- data.frame(K = 1:length(MKbar),MKbar, MKci)


save.image(file="Corr05.RData")


if(exists("cl")){
        closeCluster(cl)
	mpi.quit()
	}






#   Time:
##  Working status:
### Comments:
####Soli Deo Gloria
