#! /usr/bin/Rscript
###
### Top K test for C-elegans Data  --- leaves isolates!
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(reshape)
require(VN)

run <- FALSE

if(run){
    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    sym = TRUE
    
    if(sym){
        GE <- read.csv(file = "Achem_binsym.csv",header=FALSE)
        GF <- read.csv(file = "Agap_binsym.csv",header=FALSE)
    }else{
        GE <- read.csv(file = "Achem_asym.csv",header=FALSE)
        GF <- read.csv(file = "Agap_asym.csv",header=FALSE)
    }
    
    GE <- as.matrix(GE)
    GF <- as.matrix(GF)
    
    n <- dim(GE)[1]
    
    ### TOPK ###
    
    R <- 50;
    s <- 20; #10; # number of seeds for seeded graph matching
    g <- 0.1;
    
    Kfull <- seq(1,20,by=1) 
    lenK = length(Kfull)
    iter <- 20
    len2 <- 150 #docs ending in 500 indicate this - otherwise, used len2=300
    #J <- matrix(1,ncol=ncol(et130),nrow=nrow(et130))
    #bc <- (1/n)*J
    
    
    
    #MK <- foreach(K=1:lenK) %:% 
    MK <- foreach(le=1:len2, .combine='cbind') %dopar% {
    
            setseed <- 12345*le
            set.seed(setseed)
    
            seeds  <- sort(sample(1:n,s,replace=FALSE))
            nseeds <- setdiff(1:n,seeds)
            A   <- GE[c(seeds,nseeds),c(seeds,nseeds)]
            B   <- GF[c(seeds,nseeds),c(seeds,nseeds)]
    	
    	### Use sgm to match graphs
            Ps <- multiStart(A,B,R,s,g,iter=iter)
    	
    	rankd <- pass2ranksuplus(Ps)
    	matchtopk <- sapply(Kfull, function(x) sum(diag(rankd) <= x))
    
    	### Determine proportion of correctly matched nodes (not including seeds)	
    	prop <- (matchtopk-s)/(nrow(Ps)-s)
            #c(voiLocK, maxvoiLoc, sizenbdA, sizenbdBcorr,length(propShared))
    	prop
    }
    
    save.image(file="CelegTopKsgm12NOplot_20161126_250.RData")
}else{ 
    load("CelegTopKsgm12NOplot_20161126_250.RData")
}

MKbar <- apply(MK,1,mean)
MKsd  <- apply(MK,1,sd)/sqrt(len2)
MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2

df1 <- data.frame(Kfull,MKbar, MKci, group = "SoftSGM")
df2 <- data.frame(df1[, -5], group = "chance")

df1$group <- as.factor(df1$group)
df2$group <- as.factor(df2$group)

df2$MKbar <- Kfull/(n-s)
df2$X1 <- NA
df2$X2 <- NA

df <- rbind(df1, df2)
#df <- data.frame(Kfull,MKbar, MKci)

p04 <- 
  ggplot(data=df, aes(x=Kfull, y = MKbar, colour = group)) + 
    #ggplot(data=df, aes(x=Kfull, y = MKbar)) + 
        geom_line(size=1)  + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.7)) + 
        geom_point() + 
        theme_bw() + 
        theme(plot.background = element_blank(), 
              panel.grid.major = element_blank(),
              legend.title=element_blank(),
              panel.grid.minor = element_blank()) + 
        theme(panel.border=element_blank()) + 
        theme(axis.line.x = element_line(colour="black"),
              axis.line.y = element_line(colour="black")) +
        theme(text=element_text(size=25)) + 
        scale_colour_manual(values = c("#FF0000", "#000000")) +
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab("depth K") +
        labs(title= "C.Elegans: Depth K")

pdf("CelegTopKsgm12Plot20161126_250.pdf",height=8,width=12)
p04
dev.off()


save.image(file="CelegTopKsgm12WITHplot_20161126_250.RData")


if(run){
if(exists("cl")){
        closeCluster(cl)
	mpi.quit()
	}
}

#
##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
