#! /usr/bin/Rscript
###
### Simulation showing that runtime of SGM algorithm is cubic and is
### also a function of the underlying edge-correlation between graphs.
### Assumes a permutation (not identity) is true underlying vertex
### alignment.
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(pryr)
require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(VN)
source("../randseedselect.r")
source("../sgm.ordered.whungct.R")

run <- FALSE

if(run){
    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores) }
    
    Bern = matrix(c(0.7,0.3,0.4,0.3,0.7,0.3,0.4,0.3,0.7),3,byrow=TRUE)
    
    K = ncol(Bern)
    
    nfull1 = rev(c(50,100,150,200,250,300)+3) 
    lenn = length(nfull1);
    
    rho <- c(0.3,0.8) 
    paird <- expand.grid(n=nfull1,rho=rho)
    lenprd <- nrow(paird)
    iter <- 20
    len2 <- 25
    
    set.seed(3067)
    SS <- matrix(sample(1e6,len2*lenprd,replace=FALSE),nrow=lenprd,ncol=len2)
    s <- c(3,3,3)
    nums <- sum(s)
    print("at foreach")
    
    MK <- foreach(i=1:lenprd) %:% 
          foreach(le=1:len2, .combine='rbind') %dopar% {
            n <- rep(paird$n[i],3)
            corr <- paird$rho[i]
    
            np<-c(0,n)
            P <- matrix(0,sum(n),sum(n))
            for(ip in 1:length(n)){
                for(j in 1:length(n)){
                    P[(sum(np[1:ip])+1):sum(np[1:ip+1]),
                      (sum(np[1:j])+1):sum(np[1:j+1])] <-
                        Bern[ip,j]*matrix(1,np[ip+1],np[j+1])
                }
            }
            
            sumn <- sum(n)
            J <- matrix(1,(sumn-nums),(sumn-nums)); # J
            bc <- (1/(sumn-nums))*J #this is the barycenter
    
            setseed <- 12345*le
            set.seed(SS[i,le])
    
            ### generate random correlated matrices 
    	### -- note: Bcorr corresponds to first (nrow(B)) vertices of A
            RM <- randmatcorr(Bern,n,n,P,corr,setseed)
            seeds1 <- apply(as.matrix(n),1,function(x){sample(x[1],s[1],replace=FALSE)})
            seeds2 <- cbind(rep(0,s[1]),rep(n[1],s[1]),rep(n[1]+n[2],s[1]))
            seeds <- c(seeds1 + seeds2)
            nseeds <- setdiff(1:sumn,seeds)
            A <- RM$A[c(seeds,nseeds),c(seeds,nseeds)]
            B1 <- RM$Bc[c(seeds,nseeds),c(seeds,nseeds)]
            perm <- c(1:9,sample(c((3*s[1]+1):sumn),length(nseeds),replace=FALSE))
            pmat <- diag(1,sumn)[,perm]
            Bcorr <- B1[perm,perm]
            map <- cbind(c(1:sumn),perm)
    
            #fa <- sprintf("NrhovaryAdjMats/A%02g_%03g_%04g.txt",le,corr*100,n[1])
            #fb <- sprintf("NrhovaryAdjMats/B%02g_%03g_%04g.txt",le,corr*100,n[1])
            #fpermmap <- sprintf("NrhovaryAdjMats/map%02g_%03g_%04g.txt",le,corr*100,n[1])
            #write.table(A, file=fa, row.names=FALSE, col.names=FALSE)
            #write.table(Bcorr, file=fb, row.names=FALSE, col.names=FALSE)
            #write.table(map, file=fpermmap, row.names=FALSE, col.names=FALSE)
    	
    	### Use sgm to match graphs
            stime <- system.time(
                memdiff <- mem_change(
    		Psall <- sgm.ordered.whungct(A,Bcorr,nums,bc,pad=0,iteration=iter)
                )
            )
            memused <- mem_used()
    
            Ps <- Psall$P        
            hungit <- Psall$hungit
    
            frob <- norm(A%*%Ps - Ps%*%Bcorr,type="F")
            frobtrue <- norm(A%*%pmat - pmat%*%Bcorr,type="F")
    	### Determine proportion of correctly matched nodes (not including seeds)	
            ## if a row of Ps matches the row of pmat, then the
            ## alignment for that vertex is correct. So, the number of
            ## correct matches is the number of rows of dif that are all
            ## 0s
            dif <- Ps - pmat
            nz <- sum(apply(dif,1,function(x){all(x==0)}))
    	match = (nz-nums)/(nrow(Ps)-nums)
        #match2 <- (sum(diag(Ps[perm,]))-nums)/(nrow(Ps)-nums)
            #c(voiLocK, maxvoiLoc, sizenbdA, sizenbdBcorr,length(propShared))
    	cbind(match,stime=stime[3],cputime = stime[1]+stime[2],usertime = stime[1], systime = stime[2],memused=memused,memsgmstep=memdiff,frob=frob,hungit=hungit,frobtrue=frobtrue)#,match2)
    }
    
    save.image(file="sgm12noplot_Nrhovary.RData")
}else{
    load("sgm12noplot_Nrhovary.RData")
}

MKtimebar <- as.numeric(lapply(MK,function(x){mean(x[,2])}))
MKtimesd <- as.numeric(lapply(MK,function(x){sd(x[,2])/sqrt(len2)}))
MKtimeci <- cbind(c(MKtimebar-2*MKtimesd),c(MKtimebar+2*MKtimesd)) 

nfull <- nfull1*K - nums

dftime <- data.frame(nfull,MKtimebar, MKtimeci,as.factor(paird[,2]))
colnames(dftime) <- c("n","avg","X1","X2","rho")


rt1 <- lm(avg ~ poly(n, 3,raw=TRUE), data = dftime[dftime$rho==0.3 & !(dftime$n %in% c(60,90)),])

rt2 <- lm(avg ~ poly(n, 3,raw=TRUE), data = dftime[dftime$rho==0.8 & !(dftime$n %in% c(60,90)),])


p04t <- 
    ggplot(data=dftime, aes(x=n, y = avg, colour = rho)) + 
        geom_line(size=1, alpha = 0.5, lty = 2)  + 
        geom_smooth(method = "lm", formula = y ~ poly(x, 3), se = FALSE, size=.75,alpha=0.75, lty = 1)  +
        geom_point() + 
        theme_classic() +
        geom_errorbar(aes(ymin=X1, ymax=X2),alpha=.8) + 
        theme(text=element_text(size=25)) + 
        ylab("average runtime (s)") + 
        xlab("number of non-seed vertices (n)") +
        guides(color=guide_legend(title=expression(rho))) +
        labs(title= "Runtime (s) as n increases") +
        theme(plot.title = element_text(hjust = 0.5))

pdf("Nrhovary_elapsedRT.pdf",height=8,width=8)
p04t
dev.off()


if(run){
    if(exists("cl")){
        closeCluster(cl)
        mpi.quit()
    }
}


##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
