#! /usr/bin/Rscript
###
### Simulation for demonstrating how ratio |V1|/|V2| affects SGM
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(VN)
source("../randseedselect.r")

## NOTE TO USER: run tells whether to run full simulation. if false,
## then will simply call rdata file and generate plot.
run <- FALSE


if(run){
    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    B = matrix(c(0.7,0.3,0.4,0.3,0.7,0.3,0.4,0.3,0.7),3,byrow=TRUE)
    n = rep(100,3) #c(75,75,75)
    
    
    np<-c(0,n)
    P <- matrix(0,sum(n),sum(n))
    for(ip in 1:length(n)){
        for(j in 1:length(n)){
            P[(sum(np[1:ip])+1):sum(np[1:ip+1]),
              (sum(np[1:j])+1):sum(np[1:j+1])] <-
                B[ip,j]*matrix(1,np[ip+1],np[j+1])
        }
    }
    corr <- 0.7
    
    s <- 3 # number of seeds for seeded graph matching
    iter <- 20
    len2 <- 300 
    nc <- 16
    
    sumn <- sum(n)
    J <- matrix(1,(sumn-s),(sumn-s)); # J
    bc <- (1/(sumn-s))*J #this is the barycenter
    
    set.seed(124)
    SS <- matrix(sample(1e6,len2*nc,replace=FALSE),nrow=nc,ncol=len2)
    
    MK <- foreach(i=1:nc) %:% 
          foreach(le=1:len2, .combine='rbind') %dopar% {
            m <- floor(((20+5*i)/100)*n)
            summ <- sum(m)
    
            setseed <- 12345*le
            set.seed(SS[i,le])
    
            ### generate random correlated matrices 
    	### -- note: Bcorr corresponds to first (nrow(B)) vertices of A
            RM <- randmatcorr(B,n,m,P,corr,setseed)
            RMs <- randseedselect(RM$A,RM$Bcorr,s)
            A <- RMs$A
            Bord <- RMs$Bc

            perm <- c(1:s,sample(c((s+1):nrow(Bord)),(nrow(Bord)-s),replace=FALSE))
            pmat <- diag(1,sumn)[,perm]
            Bcorr <- Bord[perm,perm]
            map <- cbind(c(1:summ),perm)
    	
    	### Use sgm to match graphs
            Psall <- sgm.ordered(A,Bcorr,s,start=bc,maxiter=iter)
            Ps <- Psall$P        
    
    	### Determine proportion of correctly matched nodes (not including seeds)	
    	Pab = Ps[1:summ,1:summ]
    	match = (sum(diag(Pab[perm,]))-s)/(nrow(Pab)-s)
    	match
    }

    save.image(file="sgm12noplot_ratiovary.RData")
}else{
    load(file="sgm12noplot_ratiovary.RData")
}


MKbar <- as.numeric(lapply(MK,function(x){mean(x[,1])}))
MKsd <- as.numeric(lapply(MK,function(x){sd(x[,1])/sqrt(len2)}))

MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2

r <- seq(.25,1,by=.05)
chance <- rep(NA, length(r))
for(i in 1:length(r)){
    chance[i] <- 1/(sumn - 3)
}
dfp <- data.frame(r,MKbar,MKci,chance)

p04 <- 
    ggplot(data=dfp, aes(x=r, y = MKbar)) + 
        scale_y_continuous(breaks=seq(0,1,by=.1)) +
        geom_line(size=1)  + 
        geom_point() + 
        theme_classic() + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.7)) + 
        theme(text=element_text(size=25)) + 
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab("ratio r") 
#        labs(title= "Affects of Various Ratios")
    
        #geom_line(aes(x=r,y=chance),colour="red") +

pdf("sgm12plot_ratiovary_q.pdf",height=8,width=8)
p04
dev.off()




if(run){
    if(exists("cl")){
            closeCluster(cl)
    	mpi.quit()
    	}
}



#
##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
