#! /usr/bin/Rscript
###
### Simulation illustrating how the proportion of correctly matched
### vertices increases in K when using softSGM (multiStart)
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(VN)
source("../randseedselect.r")

## NOTE TO USER: run tells whether to run full simulation. if false,
## then will simply call rdata file and generate plot.
run <- FALSE


if(run){
    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    B = matrix(c(0.7,0.3,0.4,0.3,0.7,0.3,0.4,0.3,0.7),3,byrow=TRUE)
    n = rep(100,3) 
    
    np<-c(0,n)
    P <- matrix(0,sum(n),sum(n))
    for(ip in 1:length(n)){
        for(j in 1:length(n)){
            P[(sum(np[1:ip])+1):sum(np[1:ip+1]),
              (sum(np[1:j])+1):sum(np[1:j+1])] <-
                B[ip,j]*matrix(1,np[ip+1],np[j+1])
        }
    }
    corr <- 0.6
    
    s <- 3 # number of seeds for seeded graph matching
    g <- 0.1
    Rall <- c(1,seq(5,50,by=5)) 
    lenR <- length(Rall)
    len2 <- 100 
    set.seed(306)
    SS <- matrix(sample(1e6,len2*lenR,replace=FALSE),nrow=lenR,ncol=len2)
    
    MK <- foreach(i=1:lenR) %:% 
          foreach(le=1:len2, .combine='rbind') %dopar% {
#    	set.seed <- i*123+le*10000
            R <- Rall[i]
    
            setseed <- 12345*le
    
            ### generate random correlated matrices 
    	### -- note: Bcorr corresponds to first (nrow(B)) vertices of A
            set.seed(SS[i,le])
            RM <- randmatcorr(B,n,n,P,corr,setseed)
            RMs <- randseedselect(RM$A,RM$Bcorr,s)
            A <- RMs$A
            perm <- c(1:s,sample(c((s+1):nrow(A)),(nrow(A)-s),replace=FALSE))
            pmat <- diag(1,nrow(A))[,perm]
            Bcorr <- RMs$Bc[perm,perm]
            map <- cbind(c(1:nrow(A)),perm)
    	
    	#### Use softSGM to match graphs
            Ps <- multiStart(A,Bcorr,R,s,g,iter=20)
    
    	#### Determine proportion of correctly matched nodes (not including seeds)	
            
    	rankd <- pass2ranksuplus(Ps)
    	matchtop1 <- sum(diag(rankd[perm,]) <= 1)
    	match = (matchtop1-s)/(nrow(Ps)-s)
    	match
    }
    
    save.image(file="sgm12noplot_restartvary20161128_rho6.RData")
}else{
    load(file="sgm12noplot_restartvary20161128_rho6.RData")
}


MKbar <- as.numeric(lapply(MK,function(x){mean(x[,1])}))
MKsd <- as.numeric(lapply(MK,function(x){sd(x[,1])/sqrt(len2)}))
MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2

K <- Rall
df1 <- data.frame(K,MKbar, MKci)

p04 <-
    ggplot(data=df1, aes(x=K, y = MKbar)) + 
        scale_y_continuous(breaks=seq(0,1,by=0.1)) + #,limits=c(0,1)) + 
        geom_line(size=1)  + 
        geom_point() + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.7)) + 
        theme_classic() + 
        theme(text=element_text(size=25)) + 
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab("number of restarts R") +
        labs(title= "Performance of SoftSGM: Restarts") +
        theme(plot.title = element_text(hjust = 0.5))

pdf("sgm12plot_restartvary.pdf",height=8,width=8)
p04
dev.off()


#save.image(file="sgm12plot_restartvary20161128_rho6.RData")



if(run){
    if(exists("cl")){
            closeCluster(cl)
    	mpi.quit()
    	}
}




#   Time:
##  Working status:
### Comments:
####Soli Deo Gloria
