#! /usr/bin/Rscript
###
### SGM performance as a function of the number of seeds
### Enron experiment for times t=130 and t=131
### binary, symmetric with NO isolates
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(reshape)
require(VN)

run <- FALSE

if(run){

    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    
    et130 <- read.csv(file = "enront130.csv",header=FALSE)
    et131 <- read.csv(file = "enront131.csv",header=FALSE)
    et132 <- read.csv(file = "enront132.csv",header=FALSE)
    
    et130 <- as.matrix(et130)
    et131 <- as.matrix(et131)
    et132 <- as.matrix(et132)

    ## pattern recognition 201903 publication has matlab version with 
    ## bin <- sym <- noiso <- TRUE
    sym <- TRUE
    bin <- TRUE
    noiso <- TRUE

    if(sym){
        et130 <- (et130+t(et130))/2
        et131 <- (et131+t(et131))/2
        et132 <- (et132+t(et132))/2
    }
    if(bin){
        et130 <- (et130>0)+0
        et131 <- (et131>0)+0
        et132 <- (et132>0)+0
    }
    if(noiso){
        iso0 <- apply(et130,1,sum)>0
        iso1 <- apply(et131,1,sum)>0
        iso2 <- apply(et132,1,sum)>0
        iso <- ((!iso0)+(!iso2)+(!iso1))==0
        et130 <- et130[iso,iso]
        et131 <- et131[iso,iso]
        et132 <- et132[iso,iso]
    }
    
    n <- dim(et130)[1]
    
    svec <- c(0,1,5,10,20,50,60,90,100,140); #10; # number of seeds for seeded graph matching
    lens <- length(svec)
    
    iter <- 20
    len2 <- 100 #docs ending in 500 indicate this - otherwise, used len2=300
    #J <- matrix(1,ncol=ncol(et130),nrow=nrow(et130))
    #bc <- (1/n)*J
    
    #MK <- foreach(K=1:lenK) %:% 
    MK01 <- foreach(i=1:lens) %:% 
          foreach(le=1:len2, .combine='rbind') %dopar% {
    
            setseed <- 12345*le+12*i

            s <- svec[i]
            J <- matrix(1,(n-s),(n-s)); # J
            bc <- (1/(n-s))*J #this is the barycenter
    
            if(s>0){
                seeds   <- sort(sample(1:n,s,replace=FALSE))
                nseeds  <- setdiff(1:n,seeds)
                A130    <- et130[c(seeds,nseeds),c(seeds,nseeds)]
                A131ord <- et131[c(seeds,nseeds),c(seeds,nseeds)]
                #A132ord <- et132[c(seeds,nseeds),c(seeds,nseeds)]
                perm <- c(1:s,sample(c((s+1):n),(n-s),replace=FALSE))
                A131 <- A131ord[perm,perm]
                #A132 <- A132ord[perm2,perm2]
            }else{
                perm <- c(sample(1:n,n,replace=FALSE))
                #perm2 <- c(sample(1:n,n,replace=FALSE))
                A130    <- et130
                A131 <- et131[perm,perm]
                #A132 <- et132[perm2,perm2]
            }
            pmat <- diag(1,n)[,perm]
    	
    	### Use sgm to match graphs
            Ps <- sgm.ordered(A130,A131,s,bc,pad=0,maxiter=iter)
    	
    	### Determine proportion of correctly matched nodes (not including seeds)	
            #dif <- Ps$P - pmat
            #nz <- sum(apply(dif,1,function(x){all(x==0)}))
    	    #match = (nz-s)/(nrow(Ps$P)-s)
    	    rankd <- pass2ranksuplus(Ps$P)
            matched <- sum(diag(rankd[perm,])<=(1+1e-5))-s
    	    prop = (matched)/(nrow(Ps$P)-s)
    	    prop
    }
    
    
    save.image(file="Enron01_symbincon.RData")
}else{
    load("Enron01_symbincon.RData")
}


#MKbar <- apply(MK,2,mean)
#MKsd  <- apply(MK,2,sd)/sqrt(len2)
#MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2
#
#df1 <- data.frame(Kfull,MKbar, MKci, group = "SoftSGM")
#df2 <- data.frame(df1[, -5], group = "chance")
#
#df1$group <- as.factor(df1$group)
#df2$group <- as.factor(df2$group)
#
#df2$MKbar <- Kfull/(n-s)
#df2$X1 <- NA
#df2$X2 <- NA
#
#df <- rbind(df1, df2)
#
#p04 <- 
#  ggplot(data=df, aes(x=Kfull, y = MKbar, colour = group)) + 
#        geom_line(size=1)  + 
#        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.7)) + 
#        geom_point() + 
#        theme_classic() + 
#        theme(text=element_text(size=25)) + 
#        scale_colour_manual(values = c("#FF0000", "#000000")) +
#        ylab(expression(paste("average match ratio ",bar(delta)))) + 
#        xlab("depth K") +
#        labs(title= "Enron: Depth K") +
#        theme(plot.title = element_text(hjust = 0.5))
#
#pdf("EnronTopKsgm12Plot20161126_250.pdf",height=8,width=12)
#p04
#dev.off()
#
#
#save.image(file="EnronTopKsgm12WITHplot_20161126_250.RData")

if(run){
if(exists("cl")){
        closeCluster(cl)
	mpi.quit()
	}
}

#
##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
