
## Binary Symmetric and no isolates:
require(ggplot2)

load("Enron01_symbincon.RData")
load("Enron02_symbincon.RData")
load("Enron12_symbincon.RData")

MK01bar <- unlist(lapply(MK01,mean))
MK01sd  <- unlist(lapply(MK01,sd))/sqrt(len2)
MK01ci <- cbind(c(MK01bar-MK01sd),c(MK01bar+MK01sd)) # using 1 sd because SA did.

MK02bar <- unlist(lapply(MK02,mean))
MK02sd  <- unlist(lapply(MK02,sd))/sqrt(len2)
MK02ci <- cbind(c(MK02bar-MK02sd),c(MK02bar+MK02sd)) # using 1 sd because SA did.

MK12bar <- unlist(lapply(MK12,mean))
MK12sd  <- unlist(lapply(MK12,sd))/sqrt(len2)
MK12ci <- cbind(c(MK12bar-MK12sd),c(MK12bar+MK12sd)) # using 1 sd because SA did.

chancebar <- unlist(apply(as.matrix(svec),1,function(x){1/(n-x)}))

seeds <- rep(svec,4)
matching <- c( rep("130&131",length(MK01bar)), rep("131&132",length(MK12bar)),
  rep("130&132",length(MK02bar)),rep("chance",length(MK01bar)))
MKbar <- c(MK01bar,MK12bar,MK02bar,chancebar)
MKci <- rbind(MK01ci,MK12ci,MK02ci,matrix(NA,nrow=length(svec),ncol=2))

df1 <- data.frame(seeds,MKbar, MKci, group = matching)
#df2 <- data.frame(df1[, -4], group = "chance") ## check on the "-5"

df1$group <- as.factor(df1$group)
#df2$group <- as.factor(df2$group)

#df2$MKbar <- apply(as.matrix(svec),1,function(x){1/(n-x)})
#df2$X1 <- NA
#df2$X2 <- NA

#df <- rbind(df1, df2)

p04 <- 
  ggplot(data=df1, aes(x=seeds, y = MKbar, colour = group)) + 
        geom_line(size=.4)  + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.5)) + 
        geom_point() + 
        theme_classic() + 
        theme(text=element_text(size=25)) + 
#        scale_colour_manual(values = c("#FF0000", "#000000")) +
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab(expression(paste("number of seeds ",m))) 

pdf("Enron8_SA.pdf",height=8,width=12)
#pdf("Enron8_SA_1sd.pdf",height=8,width=12)
p04
dev.off()

rm(list=ls())

## Lin Lin experiment symmetric, but not binary and with isolates

require(ggplot2)

load("Enron01_LL.RData")
load("Enron02_LL.RData")
load("Enron12_LL.RData")

MK01bar <- unlist(lapply(MK01,mean))
MK01sd  <- unlist(lapply(MK01,sd))/sqrt(len2)
MK01ci <- cbind(c(MK01bar-MK01sd),c(MK01bar+MK01sd)) # using 1 sd because SA did.

MK02bar <- unlist(lapply(MK02,mean))
MK02sd  <- unlist(lapply(MK02,sd))/sqrt(len2)
MK02ci <- cbind(c(MK02bar-MK02sd),c(MK02bar+MK02sd)) # using 1 sd because SA did.

MK12bar <- unlist(lapply(MK12,mean))
MK12sd  <- unlist(lapply(MK12,sd))/sqrt(len2)
MK12ci <- cbind(c(MK12bar-MK12sd),c(MK12bar+MK12sd)) # using 1 sd because SA did.

chancebar <- unlist(apply(as.matrix(svec),1,function(x){1/(n-x)}))

seeds <- rep(svec,4)
matching <- c( rep("130&131",length(MK01bar)), rep("131&132",length(MK12bar)),
  rep("130&132",length(MK02bar)),rep("chance",length(MK01bar)))
MKbar <- c(MK01bar,MK12bar,MK02bar,chancebar)
MKci <- rbind(MK01ci,MK12ci,MK02ci,matrix(NA,nrow=length(svec),ncol=2))

df1 <- data.frame(seeds,MKbar, MKci, group = matching)
#df2 <- data.frame(df1[, -4], group = "chance") ## check on the "-5"

df1$group <- as.factor(df1$group)
#df2$group <- as.factor(df2$group)

#df2$MKbar <- apply(as.matrix(svec),1,function(x){1/(n-x)})
#df2$X1 <- NA
#df2$X2 <- NA

#df <- rbind(df1, df2)

p04 <- 
  ggplot(data=df1, aes(x=seeds, y = MKbar, colour = group)) + 
        geom_line(size=.4)  + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.5)) + 
        geom_point() + 
        theme_classic() + 
        theme(text=element_text(size=25)) + 
#        scale_colour_manual(values = c("#FF0000", "#000000")) +
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab(expression(paste("number of seeds ",m))) 

pdf("Enron8_LL_1se.pdf",height=8,width=12)
p04
dev.off()


rm(list=ls())

## HGP experiment raw: asymetric, binary (by design), and with isolates

require(ggplot2)

load("Enron01_hp.RData")
load("Enron02_hp.RData")
load("Enron12_hp.RData")

MK01bar <- unlist(lapply(MK01,mean))
MK01sd  <- unlist(lapply(MK01,sd))/sqrt(len2)
MK01ci <- cbind(c(MK01bar-MK01sd),c(MK01bar+MK01sd)) # using 1 sd because SA did.

MK02bar <- unlist(lapply(MK02,mean))
MK02sd  <- unlist(lapply(MK02,sd))/sqrt(len2)
MK02ci <- cbind(c(MK02bar-MK02sd),c(MK02bar+MK02sd)) # using 1 sd because SA did.

MK12bar <- unlist(lapply(MK12,mean))
MK12sd  <- unlist(lapply(MK12,sd))/sqrt(len2)
MK12ci <- cbind(c(MK12bar-MK12sd),c(MK12bar+MK12sd)) # using 1 sd because SA did.

chancebar <- unlist(apply(as.matrix(svec),1,function(x){1/(n-x)}))

seeds <- rep(svec,4)
matching <- c( rep("130&131",length(MK01bar)), rep("131&132",length(MK12bar)),
  rep("130&132",length(MK02bar)),rep("chance",length(MK01bar)))
MKbar <- c(MK01bar,MK12bar,MK02bar,chancebar)
MKci <- rbind(MK01ci,MK12ci,MK02ci,matrix(NA,nrow=length(svec),ncol=2))

df1 <- data.frame(seeds,MKbar, MKci, group = matching)
#df2 <- data.frame(df1[, -4], group = "chance") ## check on the "-5"

df1$group <- as.factor(df1$group)
#df2$group <- as.factor(df2$group)

#df2$MKbar <- apply(as.matrix(svec),1,function(x){1/(n-x)})
#df2$X1 <- NA
#df2$X2 <- NA

#df <- rbind(df1, df2)

p04 <- 
  ggplot(data=df1, aes(x=seeds, y = MKbar, colour = group)) + 
        geom_line(size=.4)  + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.5)) + 
        geom_point() + 
        theme_classic() + 
        theme(text=element_text(size=25)) + 
#        scale_colour_manual(values = c("#FF0000", "#000000")) +
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab(expression(paste("number of seeds ",m))) 

pdf("Enron8_hp_1se.pdf",height=8,width=12)
p04
dev.off()
