#! /usr/bin/Rscript
###
### Top K experiment for Enron Data comparing pair of graphs at times
### t=130 and t=132
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(reshape)
require(VN)

run <- FALSE

if(run){

    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    
    et130 <- read.csv(file = "enront130.csv",header=FALSE)
    #et131 <- read.csv(file = "enront131.csv",header=FALSE)
    et132 <- read.csv(file = "enront132.csv",header=FALSE)
    
    et130 <- as.matrix(et130)
    et132 <- as.matrix(et132)

    ## pattern recognition 201903 publication has 
    ## binsym <- noiso <- FALSE
    binsym <- TRUE
    noiso <- TRUE

    if(binsym){
        et130 <- ((et130+t(et130))>0)+0
        et132 <- ((et132+t(et132))>0)+0
    }
    if(noiso){
        iso0 <- apply(et130,1,sum)>0
        iso2 <- apply(et132,1,sum)>0
        iso <- ((!iso0)+(!iso2))==0
        et130 <- et130[iso,iso]
        et132 <- et132[iso,iso]
    }
    
    n <- dim(et130)[1]
    
    ### TOPK ###
    
    R <- 50;
    s <- 5; #10; # number of seeds for seeded graph matching
    g <- 0.1;
    
    Kfull <- seq(1,20,by=1) 
    lenK = length(Kfull)
    iter <- 20
    len2 <- 250 #docs ending in 500 indicate this - otherwise, used len2=300
    #J <- matrix(1,ncol=ncol(et130),nrow=nrow(et130))
    #bc <- (1/n)*J
    
    #MK <- foreach(K=1:lenK) %:% 
    MK <- foreach(le=1:len2, .combine='rbind') %dopar% {
    
            setseed <- 12345*le
    
            seeds   <- sort(sample(1:n,s,replace=FALSE))
            nseeds  <- setdiff(1:n,seeds)
            A130    <- et130[c(seeds,nseeds),c(seeds,nseeds)]
            A132ord <- et132[c(seeds,nseeds),c(seeds,nseeds)]
            perm <- c(1:s,sample(c((s+1):n),(n-s),replace=FALSE))
            pmat <- diag(1,n)[,perm]
            A132 <- A132ord[perm,perm]
    	
    	### Use sgm to match graphs
            Ps <- multiStart(A130,A132,R,s,g,iter=iter)
    	
    	rankd <- pass2ranksuplus(Ps)
    	matchtopk <- sapply(Kfull, function(x) sum(diag(rankd[perm,]) <= x))
    
    	### Determine proportion of correctly matched nodes (not including seeds)	
    	prop = (matchtopk-s)/(nrow(Ps)-s)
            #c(voiLocK, maxvoiLoc, sizenbdA, sizenbdBcorr,length(propShared))
    	prop
    }
    
    
    save.image(file="EnronTopKsgm12NOplot_20161126_250.RData")
}else{
    load("EnronTopKsgm12NOplot_20161126_250.RData")
}


MKbar <- apply(MK,2,mean)
MKsd  <- apply(MK,2,sd)/sqrt(len2)
MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2

df1 <- data.frame(Kfull,MKbar, MKci, group = "SoftSGM")
df2 <- data.frame(df1[, -5], group = "chance")

df1$group <- as.factor(df1$group)
df2$group <- as.factor(df2$group)

df2$MKbar <- Kfull/(n-s)
df2$X1 <- NA
df2$X2 <- NA

df <- rbind(df1, df2)

p04 <- 
  ggplot(data=df, aes(x=Kfull, y = MKbar, colour = group)) + 
        geom_line(size=1)  + 
        geom_errorbar(aes(ymin=X1, ymax=X2,aes=.7)) + 
        geom_point() + 
        theme_classic() + 
        theme(text=element_text(size=25)) + 
        scale_colour_manual(values = c("#FF0000", "#000000")) +
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab("depth K") +
        labs(title= "Enron: Depth K") +
        theme(plot.title = element_text(hjust = 0.5))

pdf("EnronTopK_binsym.pdf",height=8,width=12)
p04
dev.off()


save.image(file="EnronTopKsgm12WITHplot_20161126_250.RData")


if(run){
if(exists("cl")){
        closeCluster(cl)
	mpi.quit()
	}
}

#
##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
