###
### Celegans alignment of vertices with same label using SGM
###
### Heather Gaddy Patsolic 
### S.D.G 
#

require(igraph)
require(clue)
require(raster)
require(foreach)
require(ggplot2)
require(reshape)
require(VN)

run <- FALSE

if(run){
    #require(doMC); nCores <- 4
    require(doMPI)
    
    ## Register Cores for parallelism 
    if("package:doMPI" %in% search()){
    	cl <- startMPIcluster()
    	registerDoMPI(cl)} else {
    	registerDoMC(nCores)
    	}
    
    ##load('herm_Graph.Rd')
    load("celegansGraph.Rd")
    
    symbin = TRUE #symmetric, binary
    lpfree = TRUE #loop-free
    
    if(symbin){
        GE <- read.csv(file = "Achem_binsym.csv",header=FALSE)
        GF <- read.csv(file = "Agap_binsym.csv",header=FALSE)
    }else{
        GE <- read.csv(file = "Achem_asym.csv",header=FALSE)
        GF <- read.csv(file = "Agap_asym.csv",header=FALSE)
    }
    
    GE <- as.matrix(GE)
    GF <- as.matrix(GF)
    
    if(lpfree){
        diag(GE) <- 0
        diag(GF) <- 0
    }
    
    # remove isolates
    remiso <- TRUE
    
        gerowsum <- apply(GE,1,sum)
        gfrowsum <- apply(GF,1,sum)
        
        n <- dim(GE)[1]
        any(gerowsum==0) #FALSE don't need to check further
        
    if(remiso){
        locisolateF <- which(gfrowsum==0)
        noniso <- setdiff(1:n,locisolateF)
        GE <- GE[noniso,noniso]
        GF <- GF[noniso,noniso]
        n <- dim(GE)[1]
        label <- vcols[noniso]
    }
    
    
    ### SEEDS ###
    
    #R <- 50;
    #g <- 0.1;
    iter <- 20
    len2 <- 150 #250 #docs ending in 500 indicate this - otherwise, used len2=300 
    sall <- c(0,1,5,10,20,50,75,100,150,200) 
    lens <- length(sall)
    #mr <- seq(.25,1,by=.05)
    #mrn <- floor(mr*n) 
    #nc <- length(mr)
    
    set.seed(316)
    SS <- sample(1e6,lens*len2,replace=FALSE)
    setseed <- matrix(SS,nrow=lens,ncol=len2)
    
    
    MK <- foreach(i=1:lens) %:% 
          foreach(le=1:len2, .combine='rbind') %dopar% {
            s <- sall[i]
            J <- matrix(1,n-s,n-s); # J
            bc <- (1/(n-s))*J #this is the barycenter
    
    #        setseed <- 12345*le
            set.seed(setseed[i,le])
    
            seeds  <- sort(sample(1:n,s,replace=FALSE))
            nseeds <- setdiff(1:n,seeds)
            A   <- GE[c(seeds,nseeds),c(seeds,nseeds)]
            B1   <- GF[c(seeds,nseeds),c(seeds,nseeds)]
            if(s==0){
                perm <- sample(1:n,(n-s),replace=FALSE)
            }else{ 
                perm <- c(1:s,sample(c((s+1):n),(n-s),replace=FALSE))
            }
            B <- B1[perm,perm]
    
    	### Use sgm to match graphs
            Psall <- sgm.ordered(A,B,s,start=bc,pad=0,maxiter=iter)
            Ps <- Psall$corr
            Pab <- Psall$P
    
    	### Determine proportion of correctly matched nodes (not including seeds)	
    	#match = (sum(diag(Ps))-s)/(nrow(Ps)-s)
            #match = (sum(diag(Pab))-s)/(nrow(Pab)-s)
            snvcols <- vcols[perm]
            snvend <- snvcols[(s+1):n]
            snvPs  <- snvcols[Ps]
    
    	matches = sum(snvend == snvPs)/(n-s)
            #c(voiLocK, maxvoiLoc, sizenbdA, sizenbdBcorr,length(propShared))
    	matches
    }
    
    if(remiso){
        save.image(file="CelegSeedsBlocksgm12NOplot_noiso.RData")
    }else{
        save.image(file="CelegSeedsBlocksgm12NOplot_withiso.RData")
    }
}else{
    if(remiso){
        load(file="CelegSeedsBlocksgm12NOplot_noiso.RData")
    }else{
        load(file="CelegSeedsBlocksgm12NOplot_withiso.RData")
    }
}

MKbar <- as.numeric(lapply(MK,function(x){mean(x[,1])}))
MKsd <- as.numeric(lapply(MK,function(x){sd(x[,1])/sqrt(len2)}))
MKci <- cbind(c(MKbar-2*MKsd),c(MKbar+2*MKsd)) # for 95% CI, use 1.96 instead of 2
chance <- 1/(n-sall)

df <- data.frame(sall,MKbar, MKci, chance)

p04 <- 
    ggplot(data=df, aes(x=sall, y = MKbar)) + 
        scale_y_continuous(breaks=seq(0,1,by=0.02)) +
        scale_x_continuous(breaks=seq(0,200,by=25)) +
        geom_line(size=1)  + 
        geom_point() + 
        theme_classic() + 
        geom_errorbar(aes(ymin=X1, ymax=X2, width = 5)) + 
        theme(text=element_text(size=25)) + 
        ylab(expression(paste("average match ratio ",bar(delta)))) + 
        xlab("number of seeds m") +
        labs(title= "C.Elegans: Class Type") + 
        theme(plot.title = element_text(hjust = 0.5))

if(remiso){
    pdf("CelegSeedsBlocksgm12Plot_noiso.pdf",height=8,width=8)
    p04
    dev.off()
    save.image(file="CelegSeedsBlocksgm12WITHplot_noiso.RData")
}else{
    pdf("CelegSeedsBlocksgm12Plot_withiso.pdf",height=8,width=8)
    p04
    dev.off()
    save.image(file="CelegSeedsBlocksgm12WITHplot_withiso.RData")
}


if(run){
if(exists("cl")){
        closeCluster(cl)
	mpi.quit()
	}
}

##   Time:
###  Working status:
#### Comments:
#####Soli Deo Gloria
